import unittest
from nlp2.panel import *


class TestPanel(unittest.TestCase):

    def test_Empty(self):
        def a():
            pass

        argument = function_argument_panel(a)
        self.assertEqual(len(argument), 0)

    def test_None(self):
        def a(a):
            pass

        argument = function_argument_panel(a)
        self.assertEqual(len(argument), 0)

    def test_Pred(self):
        def a(input):
            pass

        argument = function_argument_panel(a)
        self.assertEqual(len(argument), 0)

    def test_Pred(self):
        def a(input=''):
            pass

        argument = function_argument_panel(a, disable_input_panel=True)
        self.assertEqual(argument['input'], '')

        argument = function_argument_panel(a, ignore_empty=True)
        self.assertEqual(argument['input'], '')

    def test_Default(self):
        def a(c="a"):
            pass

        argument = function_argument_panel(a, disable_input_panel=True)
        self.assertEqual(argument['c'], 'a')

    def test_missing(self):
        def a(c="a"):
            pass

        missing = function_check_missing_arg(a, {'d': "a"})
        self.assertEqual(missing, ['c'])
        wrong = function_check_wrong_arg(a, {'d': "a"})
        self.assertEqual(wrong, ['d'])

        missing = function_check_missing_arg(a, {'c': "a"})
        self.assertEqual(missing, [])
        wrong = function_check_wrong_arg(a, {'c': "a"})
        self.assertEqual(wrong, [])

        class B():
            def a(self, c="a"):
                pass

        b = B()
        missing = function_check_missing_arg(b.a, {'c': "a"})
        self.assertEqual(missing, [])
        wrong = function_check_wrong_arg(b.a, {'c': "a"})
        self.assertEqual(wrong, [])

    if __name__ == '__main__':
        unittest.main()



import unittest

import pytest

from nlp2.vectorize import *


class TestVectorize(unittest.TestCase):

    @pytest.mark.skip()
    def test_doc2vec_max(self):
        import gensim
        model = gensim.models.KeyedVectors.load_word2vec_format("fileloc")
        model_size = 300
        v1 = doc2vec_max(model, model_size, "你好")
        v2 = doc2vec_max(model, model_size, "你好")
        cosine_similarity(v1, v2)
        self.assertEqual(1, int(cosine_similarity(v1, v2)))
        self.assertEqual(v1, v2)

    @pytest.mark.skip()
    def test_doc2vec_aver(self):
        import gensim
        model = gensim.models.KeyedVectors.load_word2vec_format("fileloc")
        model_size = 300
        v1 = doc2vec_aver(model, model_size, "你好")
        v2 = doc2vec_aver(model, model_size, "你好")
        cosine_similarity(v1, v2)
        self.assertEqual(1, int(cosine_similarity(v1, v2)))
        self.assertEqual(v1, v2)

    @pytest.mark.skip()
    def test_doc2vec_concat(self):
        import gensim
        model = gensim.models.KeyedVectors.load_word2vec_format("fileloc")
        model_size = 300
        v1 = doc2vec_concat(model, model_size, "你好")
        v2 = doc2vec_concat(model, model_size, "你好")
        cosine_similarity(v1, v2)
        self.assertEqual(1, int(cosine_similarity(v1, v2)))
        self.assertEqual(v1, v2)

    @pytest.mark.skip()
    def test_doc2vec_concat(self):
        import gensim
        model = gensim.models.KeyedVectors.load_word2vec_format("fileloc")
        model_size = 300
        v1 = doc2vec_hier(model, model_size, "你好", windows=2)
        v2 = doc2vec_hier(model, model_size, "你好", windows=2)
        self.assertEqual(1, int(cosine_similarity(v1, v2)))
        self.assertEqual(v1, v2)


if __name__ == '__main__':
    unittest.main()



import re
import math

punctuations = r"[．﹑︰〈〉─《﹖﹣﹂﹁﹔！？｡。＂＃＄％＆＇（）＊＋，﹐－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.．!\"#$%&()*+,\-.\:;<=>?@\[\]\\\/^_`{|}~]+"
httplink = r"(https|http)[:\/0-9a-zA-Z.?=&;]*"
unused = r":.+:|<br>|\\n|\(\w+\)|\[.+\]"


def clean_all(string):
    return clean_unused_tag(clean_httplink(clean_htmlelement(string)))


def clean_unused_tag(string):
    p = re.compile(unused)
    return p.sub(' ', string.strip()).strip()


def clean_htmlelement(data):
    p = re.compile(r'<.*?>')
    return p.sub(' ', data.strip()).strip()


def clean_httplink(string):
    p = re.compile(httplink)
    return p.sub(' ', string.strip()).strip()


def split_lines_by_punc(lines, max_len=50):
    sentences = []
    for line in lines:
        line = full2half(line.strip())
        for sentence in filter(None, re.split(punctuations, line)):
            sentence = sentence.strip()
            if len(sentence) > 0 and len(split_sentence_to_array(sentence)) < max_len:
                sentences.append(sentence)
    return sentences


def split_sentence_to_ngram(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                ngrams.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))
    return ngrams


def split_sentence_to_ngram_in_part(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        part = []
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                part.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))

        ngrams.append(part)
    return ngrams


def split_text_in_all_comb(sentence):
    result = []
    for i in range(len(sentence)):
        ipart = "".join(sentence[:i + 1])
        irest = sentence[i + 1:]
        if len(irest) > len(ipart):
            for j in split_text_in_all_comb(irest):
                result.append((ipart + " " + j).rstrip())
        else:
            result.append((ipart + " " + join_words_to_sentence(irest)).rstrip())
    return result


def split_sentence_to_array(sentence, merge_non_eng=False):
    if merge_non_eng:
        regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]+" + "|" + punctuations
    else:
        regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]" + "|" + punctuations
    path = re.findall(regex, sentence, re.UNICODE)
    return path


def join_words_to_sentence(array):
    return ''.join([subPath + (' ' if is_all_english(str(array)) else '') for subPath in array]).strip()


def passage_into_chunk(text, length):
    lines = (i.strip() for i in text.splitlines())
    result = []
    chunk = ''
    for line in lines:
        chunk += (line + " ")
        if len(chunk) > length:
            result.append(chunk)
            chunk = ''
    if len(chunk) > 0:
        result.append(chunk)
    return result


def is_all_english(s):
    try:
        s.encode(encoding='utf-8').decode('ascii')
    except UnicodeDecodeError:
        return False
    else:
        return True


def is_contain_number(s):
    return bool(re.search(r'[0-9]+', s))


def is_contain_english(s):
    return bool(re.search(r'[a-zA-Z]+', s))


def is_list_contain_string(f_str, f_list):
    return any(s in f_str for s in f_list) or any(f_str in s for s in f_list)


def full2half(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 0x3000:
            num = 32
        elif 0xFF01 <= num <= 0xFF5E:
            num -= 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def half2full(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 320:
            num = 0x3000
        elif 0x21 <= num <= 0x7E:
            num += 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def sliding_windows_larger_step(a, slide=128):
    for i in range(math.ceil(len(a) / slide) + 1):
        move = int(i * slide / 2)
        if len(a[move:move + slide]) > 0:
            yield a[move:move + slide]


def sliding_windows(a, slide=128):
    for i in range(int(len(a) / slide) + 1):
        if len(a[i * slide:i * slide + slide]) > 0:
            yield a[i * slide:i * slide + slide]


def list_in_windows(list, windows):
    if windows > len(list):
        return [list]
    iter_range = (len(list) - windows + 1)
    jump = int(iter_range / 2)
    return [list[x:x + windows] for x in range(0, iter_range, jump)]



import unittest

from nlp2.rand import *


class TestRandom(unittest.TestCase):

    def test_random_string(self):
        self.assertEqual(len(random_string(3)), 3)
        self.addTypeEqualityFunc(str, random_string(3))

    def test_random_string_with_timestamp(self):
        self.assertEqual(len(random_string_with_timestamp(3)), 10 + 3)
        self.addTypeEqualityFunc(str, random_string_with_timestamp(3))

    def test_random_value_in_array_form(self):
        self.assertEqual(random_value_in_array_form([3, 3]), 3)
        self.assertEqual(random_value_in_array_form([3.0, 3.0]), 3)
        self.assertEqual(random_value_in_array_form(['A', 'A']), 'A')


if __name__ == '__main__':
    unittest.main()



import inspect


class Panel:
    def __init__(self):
        self.element_list = []
        self.key_list = []
        self.result_dict = {}

    def add_element(self, k, v, msg, default):
        if isinstance(v, list):
            inputted = ''
            while inputted not in [str(e) for e in v]:
                inputted = input(msg + ", input an item in the list " + str(v) + ", [default=" + str(default) + "]: ")
                if inputted == '':
                    inputted = str(default)
            self.element_list.append(inputted)
        else:
            if isinstance(v, float) and 0 < v < 1:  # probability
                invalue = float(input(msg + " (between 0-1), [default=" + str(default) + "]: ") or v)
            elif isinstance(v, float):  # number
                invalue = float(input(msg + " (float), [default=" + str(default) + "]: ") or v)
            elif isinstance(v, int):  # number
                invalue = int(input(msg + " (number), [default=" + str(default) + "]: ") or v)
            else:
                invalue = input(msg + ", [default=" + str(default) + "]: ")
            self.element_list.append(invalue)
        self.key_list.append(k)

    def get_result_dict(self):
        result_dict = dict(zip(self.key_list, self.element_list))
        return {k: v for k, v in result_dict.items() if
                v is not None and len(str(v)) > 0}  # only return non empty result


def function_get_all_arg(func):
    if len(inspect.getfullargspec(func).args) > 0:
        arg_len = len(inspect.getfullargspec(func).args)
        def_len = len(inspect.getfullargspec(func).defaults)
        return inspect.getfullargspec(func).args[arg_len - def_len:]
    else:
        return []


def function_get_all_arg_with_value(func):
    if len(inspect.getfullargspec(func).args) > 0:
        arg_len = len(inspect.getfullargspec(func).args)
        def_len = len(inspect.getfullargspec(func).defaults)
        arg_w_def = dict(zip(inspect.getfullargspec(func).args[arg_len - def_len:],
                             inspect.getfullargspec(func).defaults))
        return arg_w_def
    else:
        return {}


def function_check_wrong_arg(func, input_arg):
    all_arg = function_get_all_arg(func)
    return [arg for arg in input_arg if arg not in all_arg]


def function_check_missing_arg(func, input_arg):
    all_arg = function_get_all_arg(func)
    return [arg for arg in all_arg if arg not in input_arg]


def function_argument_panel(func, inputted_arg={}, disable_input_panel=False, ignore_empty=False,
                            func_parent=None, show_func_name=False):
    """use inquirer panel to let user input function parameter or just use default value"""
    fname = func.__name__
    if len(inspect.getfullargspec(func).args) > 0 and inspect.getfullargspec(func).defaults is not None:
        def_arg = function_get_all_arg_with_value(func)
        function_def_arg = {**def_arg, **inputted_arg}
        # panel
        panel = Panel()
        for k, v in def_arg.items():
            if v is not None and (not isinstance(v, str) or len(v) > 0 or not ignore_empty):
                msg = fname + " " + k if show_func_name else k
                default_v = v
                if callable(v):
                    v = v(func_parent) if func_parent else v()
                    function_def_arg[k] = v[0]  # set default value
                    default_v = v[0]
                elif isinstance(v, bool):
                    v = [True, False]
                if not disable_input_panel:
                    panel.add_element(k, v, msg, default_v)

        if not disable_input_panel:
            function_def_arg.update(panel.get_result_dict())
        return function_def_arg
    else:
        return inputted_arg



import time
from random import sample, randint, uniform


def _checkInt(l):
    return all(isinstance(i, int) for i in l)


def _checkStr(l):
    return all(isinstance(i, str) for i in l)


def _checkFloat(l):
    return any(isinstance(i, float) for i in l)


def random_string(length):
    return ''.join(sample('0123456789ABCDEF', length))


def random_string_with_timestamp(length):
    return str(int(time.time())) + random_string(length)


def random_value_in_array_form(input_array):
    if _checkInt(input_array):
        return randint(input_array[0], input_array[1])
    elif _checkFloat(input_array):
        return uniform(input_array[0], input_array[1])
    elif _checkStr(input_array):
        return input_array[randint(0, len(input_array) - 1)]



import json
import os
import re
import sys
import urllib.request


def get_folders_form_dir(root, match=""):
    for path, subdirs, files in os.walk(root):
        if path != root:
            yield path


def get_filename_from_path(path):
    """Extracts basename of a given path. Should Work with any OS Path on any OS"""
    basename = re.search(r'[^\\/]+(?=[\\/]?$)', path)
    if basename:
        return basename.group(0)


def get_files_from_dir(root, match=""):
    for path, subdirs, files in os.walk(root):
        for file in files:
            if len(match) > 0:
                if file in match:
                    yield path + file
            else:
                yield os.path.join(path, file)


def read_dir_files_yield_lines(dir_path):
    for file in os.listdir(dir_path):
        with open(os.path.join(dir_path, file), "r", encoding="utf8") as f:
            for line in f:
                yield line.strip()


def read_dir_files_into_lines(dir_path):
    ret = []
    for file in os.listdir(dir_path):
        with open(os.path.join(dir_path, file), "r", encoding="utf8") as f:
            ret += f.read().splitlines()
    return ret


def read_files_yield_lines(path):
    with open(path, "r", encoding='utf-8') as file:
        for line in file:
            yield line.strip()


def read_files_into_lines(path):
    with open(path, "r", encoding='utf-8') as file:
        return file.read().splitlines()


def create_new_dir_always(dirPath):
    if not is_dir_exist(dirPath):
        os.makedirs(dirPath)
    else:
        file_list = os.listdir(dirPath)
        for fileName in file_list:
            os.remove(dirPath + "/" + fileName)


def get_dir_with_notexist_create(dirPath):
    if not is_dir_exist(dirPath):
        os.makedirs(dirPath)
    return dirPath


def write_json_to_file(json_str, loc):
    with open(loc, 'w', encoding='utf-8') as outfile:
        json.dump(json_str, outfile, indent=4, ensure_ascii=False)
    return loc


def is_file_exist(file_loc):
    return os.path.isfile(file_loc)


def is_dir_exist(file_dir):
    return os.path.isdir(file_dir)


def _progress(block_num, block_size, total_size):
    sys.stdout.write('\r>> Downloading %.1f%%' % (float(block_num * block_size) / float(total_size) * 100.0))
    sys.stdout.flush()


def download_file(url, outdir):
    outdir = get_dir_with_notexist_create(outdir)
    outfile = url.split('/')[-1]
    if not is_file_exist(outdir + outfile):
        try:
            urllib.request.urlretrieve(url, outdir + outfile, _progress)
        except:
            return "File not found"
    sys.stdout.write("\n")
    return outdir + outfile



import unittest
from nlp2.file import *


class TestFile(unittest.TestCase):
    ROOT_DIR = os.path.dirname(os.path.abspath(__file__ + "/../"))
    DATA_DIR = os.path.join(ROOT_DIR, 'test/')

    def test_get_filename_from_path(self):
        paths = ['a/b/c/', 'a/b/c', '\\a\\b\\c', '\\a\\b\\c\\', 'a\\b\\c', 'a/b/../../a/b/c/', 'a/b/../../a/b/c']
        for path in paths:
            self.assertTrue('c' == get_filename_from_path(path))

    def test_get_folders_form_dir(self):
        for i in get_folders_form_dir(os.path.join(TestFile.DATA_DIR, './test_folder')):
            self.assertTrue(is_dir_exist(i))

    def test_get_files_from_dir(self):
        for i in get_files_from_dir(os.path.join(TestFile.DATA_DIR, './test_folder')):
            self.assertTrue(is_file_exist(i))

    def test_read_dir_files_yield_lines(self):
        self.assertCountEqual(list(read_dir_files_yield_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text/'))), ['abcabcabc', '天氣晴朗', '多雲', 'cbcbcb'])
        self.assertCountEqual(list(read_dir_files_yield_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text'))), ['abcabcabc', '天氣晴朗', '多雲', 'cbcbcb'])

    def test_read_dir_files_into_lines(self):
        self.assertCountEqual(read_dir_files_into_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text/')), ['abcabcabc', '天氣晴朗', '多雲', 'cbcbcb'])
        self.assertCountEqual(read_dir_files_into_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text')), ['abcabcabc', '天氣晴朗', '多雲', 'cbcbcb'])

    def test_read_files_yield_lines(self):
        self.assertEqual(list(read_files_yield_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text/file1'))), ['abcabcabc', '天氣晴朗', '多雲'])

    def test_read_files_into_lines(self):
        self.assertEqual(read_files_into_lines(os.path.join(TestFile.DATA_DIR, './test_folder/text/file1')), ['abcabcabc', '天氣晴朗', '多雲'])

    def test_create_new_dir_always(self):
        create_new_dir_always(os.path.join(TestFile.DATA_DIR, './test_folder/new'))
        self.assertTrue(is_dir_exist(os.path.join(TestFile.DATA_DIR, './test_folder/new')))

    def test_get_dir_with_notexist_create(self):
        get_dir_with_notexist_create(os.path.join(TestFile.DATA_DIR, './test_folder/new2'))
        self.assertTrue(is_dir_exist(os.path.join(TestFile.DATA_DIR, './test_folder/new2')))

    def test_write_json_to_file(self):
        write_json_to_file({"sent": "hi"}, os.path.join(TestFile.DATA_DIR, './test_folder/test.json'))
        self.assertTrue(is_file_exist(os.path.join(TestFile.DATA_DIR, './test_folder/test.json')))

    def test_is_file_exist(self):
        self.assertTrue(is_file_exist(os.path.join(TestFile.DATA_DIR, './test_folder/text/file1')))
        self.assertFalse(is_file_exist(os.path.join(TestFile.DATA_DIR, './test_folder/text/file100')))

    def test_is_dir_exist(self):
        self.assertTrue(is_dir_exist(os.path.join(TestFile.DATA_DIR, './test_folder/text')))
        self.assertFalse(is_dir_exist(os.path.join(TestFile.DATA_DIR, './test_folder/test')))

    def test_download_file(self):
        i = download_file(
            'https://raw.githubusercontent.com/voidful/voidful_blog/master/assets/post_src/nninmath_3/img1',
            os.path.join(TestFile.DATA_DIR, './test_folder/'))
        self.assertEqual(i, os.path.join(TestFile.DATA_DIR, './test_folder/img1'))

    def test_download_file_not_found(self):
        i = download_file(
            'https://dumps.wikimedia.org/abc/latest/', os.path.join(TestFile.DATA_DIR, './test_folder/'))
        self.assertEqual(i, "File not found")


if __name__ == '__main__':
    unittest.TestFile()
